#ifndef ASSERT
#define ASSERT
#include <assert.h>
#endif

#ifndef COMMONS
#define COMMONS
#include "commons.c"
#endif

#ifndef T_LISTS
#define T_LISTS
#include "t_lists.c"
#endif

#ifndef ARRAYS
#define ARRAYS
#include "arrays.c"
#endif

typedef struct Struct_ArrayDeque {
	int MIN_INITIAL_CAPACITY;
	int head;
	int tail;
	PObject* elements;
	int elements_length;
} ArrayDeque;

typedef ArrayDeque* PArrayDeque;

bool arraydeque_invariant_1(PArrayDeque arraydeque) {
	return arraydeque->elements != NULL;
}

bool arraydeque_invariant_2(PArrayDeque arraydeque) {
	return 0 <= arraydeque->head
			&& arraydeque->head < arraydeque->elements_length;
}

bool arraydeque_invariant_3(PArrayDeque arraydeque) {
	return 0 <= arraydeque->tail
			&& arraydeque->tail < arraydeque->elements_length;
}

bool arraydeque_invariant_4(PArrayDeque arraydeque) {
	return arraydeque->elements[arraydeque->tail] == NULL;
}

bool arraydeque_invariant_5(PArrayDeque arraydeque) {
	return arraydeque->MIN_INITIAL_CAPACITY == 8;
}

bool arraydeque_invariant_6(PArrayDeque arraydeque) {
	return !(arraydeque->head < arraydeque->tail)
			|| t_lists_neq(arraydeque->elements, arraydeque->head,
					arraydeque->tail, NULL);
}

bool arraydeque_invariant_7(PArrayDeque arraydeque) {
	return !(arraydeque->head < arraydeque->tail)
			|| t_lists_eq(arraydeque->elements, 0, arraydeque->head, NULL);
}

bool arraydeque_invariant_8(PArrayDeque arraydeque) {
	return !(arraydeque->head < arraydeque->tail)
			|| t_lists_eq(arraydeque->elements, arraydeque->tail,
					arraydeque->elements_length, NULL);
}

bool arraydeque_invariant_9(PArrayDeque arraydeque) {
	return !(arraydeque->tail < arraydeque->head)
			|| t_lists_neq(arraydeque->elements, 0, arraydeque->tail, NULL);
}

bool arraydeque_invariant_10(PArrayDeque arraydeque) {
	return !(arraydeque->tail < arraydeque->head)
			|| t_lists_neq(arraydeque->elements, arraydeque->head,
					arraydeque->elements_length, NULL);
}

bool arraydeque_invariant_11(PArrayDeque arraydeque) {
	return !(arraydeque->tail < arraydeque->head)
			|| t_lists_eq(arraydeque->elements, arraydeque->tail,
					arraydeque->head, NULL);
}

bool arraydeque_invariant_12(PArrayDeque arraydeque) {
	return !(arraydeque->tail == arraydeque->head)
			|| t_lists_eq(arraydeque->elements, 0, arraydeque->elements_length,
					NULL);
}

bool arraydeque_invariants(PArrayDeque arraydeque) {
	return arraydeque_invariant_1(arraydeque)
			&& arraydeque_invariant_2(arraydeque)
			&& arraydeque_invariant_3(arraydeque)
			&& arraydeque_invariant_4(arraydeque)
			&& arraydeque_invariant_5(arraydeque)
			&& arraydeque_invariant_6(arraydeque)
			&& arraydeque_invariant_7(arraydeque)
			&& arraydeque_invariant_8(arraydeque)
			&& arraydeque_invariant_9(arraydeque)
			&& arraydeque_invariant_10(arraydeque)
			&& arraydeque_invariant_11(arraydeque)
			&& arraydeque_invariant_12(arraydeque);
}
/*-----------------------------------------------------------------------------
 * removeFirstOccurrence(Object value)
 *-----------------------------------------------------------------------------
 */
void arraydeque_checkInvariants(PArrayDeque arraydeque, int* error_code)
{
	bool cond1 = arraydeque->elements[arraydeque->tail]==NULL;
	if (!cond1) {
		(*error_code) = ERROR;
		return;
	}

	bool cond2;
	if (arraydeque->head==arraydeque->tail) {
		cond2 = arraydeque->elements[arraydeque->head]==NULL;
	} else {
		cond2 = arraydeque->elements[arraydeque->head]!=NULL && arraydeque->elements[(arraydeque->tail-1) & (arraydeque->elements_length-1)]!=NULL;
	}
	if (!cond2) {
		(*error_code) = ERROR;
		return;
	}

	bool cond3 = arraydeque->elements[(arraydeque->head-1) & (arraydeque->elements_length-1)]==NULL;
	if (!cond3) {
		(*error_code) = ERROR;
		return;
	}

}

bool arraydeque_delete(PArrayDeque arraydeque, int index, PErrorCode error_code)
{
	arraydeque_checkInvariants(arraydeque, error_code);
	if (*error_code==ERROR)
		return FALSE;

	PObject* elements = arraydeque->elements;
	int elements_length = arraydeque->elements_length;
	int mask = elements_length -1;
	int h = arraydeque->head;
	int t = arraydeque->tail;
	int front = (index-h) & mask;
	int back = (t-h)& mask;
	if (front >= ((t-h)& mask)) {
		(*error_code)=ERROR;
		return FALSE;
	}

	if (front<back) {
		if (h<=index) {
			system_arraycopy(elements, elements_length, h, elements, elements_length, h+1,front);
		} else {
			system_arraycopy(elements, elements_length, 0, elements, elements_length, 1, index);
			elements[0] = elements[mask];
			system_arraycopy(elements, elements_length, h, elements, elements_length, h+1, mask-h);
		}
		elements[h]=NULL;
		arraydeque->head = (h+1)&mask;
		return FALSE;
	} else {
		if (index<t) {
			system_arraycopy(elements, elements_length, index+1, elements, elements_length, index, back);
			arraydeque->tail = t -1;
		} else {
			system_arraycopy(elements, elements_length, index+1, elements, elements_length, index, mask);
			elements[mask]= elements[0];
			system_arraycopy(elements, elements_length, 1, elements, elements_length, 0,t);
			arraydeque->tail = (t-1)& mask;
		}
		return TRUE;
	}

}

/*-----------------------------------------------------------------------------
 * contains(Object value)
 *-----------------------------------------------------------------------------
 */

bool arraydeque_contains(PArrayDeque arraydeque, PObject value) {
	if (value == NULL)
		return FALSE;

	int ic = arraydeque->head;

	//assert (arraydeque->elements!=NULL);	//CRASH:4.072s
	//assert (ic>=0);	//CRASH:4.067s
	//assert (ic<arraydeque->elements_length);	//SAFE:1m10.661s
	PObject x = arraydeque->elements[ic];
	while (x != NULL) {
		if (value == x) {
			break;
		}
		//	assert (arraydeque->elements!=NULL);	//CRASH:5.22s
		if (ic == arraydeque->elements_length) {
			ic = 0;
		} else {
			ic++;
		}
		//assert (arraydeque->elements!=NULL);	//SAFE:27.101s
		//assert (ic>=0);	//SAFE:41.734s
		//assert (ic<arraydeque->elements_length);	//UNSAFE:14.734s
		x = arraydeque->elements[ic];
	}
	if (x != NULL)
		return TRUE;
	else
		return FALSE;
}

void blast_arraydeque_contains(PArrayDeque arraydeque, PObject value) {
	if (arraydeque == NULL)
		return;

	if (!arraydeque_invariants(arraydeque)) {
		return;
	}

	bool result= arraydeque_contains(arraydeque, value);

//	assert(result==TRUE || (value==NULL || !t_lists_deqhas(arraydeque->elements, arraydeque->elements_length, arraydeque->head, arraydeque->tail, arraydeque->tail, value)));	//UNSAFE:2m4.608s
	//assert(result==FALSE || (value!=NULL && t_lists_deqhas(arraydeque->elements, arraydeque->elements_length, arraydeque->head, arraydeque->tail, arraydeque->tail, value)));	//UNSAFE:2m5.257s

//	assert (arraydeque_invariant_1(arraydeque));	//CRASH:5.124s
//	assert (arraydeque_invariant_2(arraydeque));	//CRASH:4.465s
//	assert (arraydeque_invariant_3(arraydeque));	//CRASH:4.545s
//	assert (arraydeque_invariant_4(arraydeque));	//CRASH:4.841s
//	assert (arraydeque_invariant_5(arraydeque));	//CRASH:4.555s
//	assert (arraydeque_invariant_6(arraydeque));	//CRASH:5.375s
//	assert (arraydeque_invariant_7(arraydeque));	//CRASH:4.598s
//	assert (arraydeque_invariant_8(arraydeque));	//UNSAFE:32.866s
//	assert (arraydeque_invariant_9(arraydeque));	//CRASH:4.999s
//	assert (arraydeque_invariant_10(arraydeque));	//UNSAFE:1m57s
//	assert (arraydeque_invariant_11(arraydeque));	//CRASH:4.997s
//	assert (arraydeque_invariant_12(arraydeque));	//UNSAFE:8.18s
}

/*-----------------------------------------------------------------------------
 * removeFirstOccurrence(Object value)
 *-----------------------------------------------------------------------------
 */
bool arraydeque_removeFirstOccurrence(PArrayDeque arraydeque, PObject value, PErrorCode error_code)
{
	if (value==NULL)
		return FALSE;

	int ic = arraydeque->head;

//	assert (arraydeque->elements!=NULL);	//SAFE:23.96s
//	assert (ic>=0); //SAFE:1m8s
//	assert (ic<arraydeque->elements_length); //SAFE:1m43.467s
	PObject x = arraydeque->elements[ic];

	while(x!=NULL) {
		if (value==x) {
			break;
		}
//		assert (arraydeque->elements!=NULL);	//SAFE:24s
		if (ic==arraydeque->elements_length) {
			ic=0;
		} else {
			ic++;
		}
//		assert (arraydeque->elements!=NULL);	//SAFE:43.740s
//		assert (ic>=0); //SAFE:1m12s
//		assert (ic<arraydeque->elements_length); //UNSAFE:16.87s
		x = arraydeque->elements[ic];
	}

	if (x!=NULL) {
//		assert (ic>=0 && ic<arraydeque->elements_length);	//UNSAFE:1m49s
//		assert (t_lists_deqwithin(arraydeque->elements, arraydeque->elements_length, arraydeque->head, arraydeque->tail, ic));	//UNSAFE:1m10s
		arraydeque_delete(arraydeque, ic, error_code);
		return TRUE;
	} else {
		return FALSE;
	}
}

void blast_arraydeque_removeFirstOccurrence(PArrayDeque arraydeque, PObject value) {
	ErrorCode error_code = OK;

	if (arraydeque == NULL)
		return;

	if (!arraydeque_invariants(arraydeque)) {
		return;
	}

	PObject* old_elements = arrays_clone(arraydeque->elements, arraydeque->elements_length);
	int old_elements_length = arraydeque->elements_length;
	int old_head = arraydeque->head;
	int old_tail = arraydeque->tail;

	bool result= arraydeque_removeFirstOccurrence(arraydeque, value, &error_code);

	if (error_code==OK) {
//		assert (result==TRUE || (value==NULL || (!t_lists_deqhas(arraydeque->elements, arraydeque->elements_length, arraydeque->head, arraydeque->tail, arraydeque->tail, value) && arraydeque->head==old_head && arraydeque->tail==old_tail && t_arrays_object_equals_2(arraydeque->elements, old_elements, 0, arraydeque->elements_length))));	//UNSAFE:33s
	}

	if (error_code==OK) {
//		assert (result==FALSE || (value!=NULL && t_lists_deqhas(old_elements, old_elements_length, old_head, old_tail, old_tail, value)));	//UNSAFE:18s
	}

//		assert (arraydeque_invariant_1(arraydeque));	//SAFE:25.193s
//		assert (arraydeque_invariant_2(arraydeque));	//CRASH:4.465s
//		assert (arraydeque_invariant_3(arraydeque));	//CRASH:4.545s
//		assert (arraydeque_invariant_4(arraydeque));	//CRASH:4.841s
//		assert (arraydeque_invariant_5(arraydeque));	//CRASH:4.555s
//		assert (arraydeque_invariant_6(arraydeque));	//CRASH:5.375s
//		assert (arraydeque_invariant_7(arraydeque));	//CRASH:4.598s
//		assert (arraydeque_invariant_8(arraydeque));	//UNSAFE:32.866s
//		assert (arraydeque_invariant_9(arraydeque));	//CRASH:4.999s
//		assert (arraydeque_invariant_10(arraydeque));	//UNSAFE:1m57s
//		assert (arraydeque_invariant_11(arraydeque));	//CRASH:4.997s
//		assert (arraydeque_invariant_12(arraydeque));	//UNSAFE:8.18s
}

/*-----------------------------------------------------------------------------
 * removeLastOccurrence(Object value)
 *-----------------------------------------------------------------------------
 */
bool arraydeque_removeLastOccurrence(PArrayDeque arraydeque, PObject value, PErrorCode error_code)
{
	if (value==NULL)
		return FALSE;

	int indexToRemove = -1;
	if (arraydeque->head < arraydeque->tail) {
		int ic = arraydeque->tail -1;
		for ( ; ic>=arraydeque->head; ic--) {
//			assert (arraydeque->elements!=NULL);	//SAFE:2m38s
//			assert (ic>=0);	//SAFE:4m20s
//			assert (ic<arraydeque->elements_length);	//SAFE:4m20s
			if (value==arraydeque->elements[ic]) {
				break;
			}
		}
		if (ic>=arraydeque->head) {
			indexToRemove = ic;
		}
	} else {
		int ic = arraydeque->tail -1;
		for ( ; ic>=0 ; ic--) {
//			assert (arraydeque->elements!=NULL);	//SAFE:2m10s
//			assert (ic>=0);	//SAFE:25s
//			assert (ic<arraydeque->elements_length);	//SAFE:25s
			if (value==arraydeque->elements[ic]) {
				break;
			}
		}
		if (ic>=0) {
			indexToRemove = ic;
		} else {
//			assert (arraydeque->elements!=NULL);	//SAFE:25s
			ic = arraydeque->elements_length - 1;
			for (;ic>=arraydeque->head;ic--) {
//				assert (arraydeque->elements!=NULL);	//SAFE:26s
//				assert (ic>=0);	//SAFE:37s
//				assert (ic<arraydeque->elements_length);	//SAFE:37s
				if (value==arraydeque->elements[ic]) {
					break;
				}
			}
			if (ic>=arraydeque->head) {
				indexToRemove = ic;
			}
		}
	}
	if (indexToRemove!=-1) {
//		assert (indexToRemove>=0 && indexToRemove<arraydeque->elements_length);	//CRASH:2m57s
//		assert (t_lists_deqwithin(arraydeque->elements, arraydeque->elements_length, arraydeque->head, arraydeque->tail, indexToRemove));	//CRASH:1m33.006s
		arraydeque_delete(arraydeque, indexToRemove, error_code);
		if ((*error_code)==ERROR) {
			return FALSE;
		}

		return TRUE;
	} else {
		return FALSE;
	}

}

void blast_arraydeque_removeLastOccurrence(PArrayDeque arraydeque, PObject value)
{
	ErrorCode error_code = OK;
	if (arraydeque == NULL)
		return;

	if (!arraydeque_invariants(arraydeque)) {
		return;
	}

	PObject* old_elements = arrays_clone(arraydeque->elements, arraydeque->elements_length);
	int old_elements_length = arraydeque->elements_length;
	int old_head = arraydeque->head;
	int old_tail = arraydeque->tail;

	bool result= arraydeque_removeLastOccurrence(arraydeque, value, &error_code);

	if (error_code==OK) {
//		assert (result==TRUE || (value==NULL || (!t_lists_deqhas(arraydeque->elements, arraydeque->elements_length, arraydeque->head, arraydeque->tail, arraydeque->tail, value) && arraydeque->head==old_head && arraydeque->tail==old_tail && t_arrays_object_equals_2(arraydeque->elements, old_elements, 0, arraydeque->elements_length) ) ));	//UNSAFE:4m
//		assert (result==FALSE || (value!=NULL && t_lists_deqhas(old_elements, old_elements_length, old_head, old_tail, old_tail, value)));	//CRASH:1.645
//		assert (arraydeque->head >= arraydeque->tail || t_lists_neq(arraydeque->elements, arraydeque->head, arraydeque->tail, NULL));	//UNSAFE:1m37s
	}



//			assert (arraydeque_invariant_1(arraydeque));	//SAFE:25s
//			assert (arraydeque_invariant_2(arraydeque));	//TIMEOUT:10m
//			assert (arraydeque_invariant_3(arraydeque));	//SAFE:41s
//			assert (arraydeque_invariant_4(arraydeque));	//UNSAFE:15s
//			assert (arraydeque_invariant_5(arraydeque));	//SAFE:23s
//			assert (arraydeque_invariant_6(arraydeque));	//UNSAFE:2m
//			assert (arraydeque_invariant_7(arraydeque));	//CRASH:3m
//			assert (arraydeque_invariant_8(arraydeque));	//UNSAFE:2m11s
//			assert (arraydeque_invariant_9(arraydeque));	//UNSAFE:7m
//			assert (arraydeque_invariant_10(arraydeque));	//CRASH:3m23s
//			assert (arraydeque_invariant_11(arraydeque));	//UNSAFE:1m34s
//			assert (arraydeque_invariant_12(arraydeque));	//UNSAFE:14s

}
