#ifndef ASSERT
#define ASSERT
#include <assert.h>
#endif

#ifndef COMMONS
#define COMMONS
#include "commons.c"
#endif

#ifndef T_ARRAYS
#define T_ARRAYS
#include "t_arrays.c"
#endif

typedef struct struc_PartitionIndexes {
	int a;
	int b;
	int c;
	int d;
} PartitionIndexes;

typedef PartitionIndexes* PPartitionIndexes;
/*
 * --------------------------------------------------------------
 * Auxiliary Functions
 * --------------------------------------------------------------
 */

void arrays_swap(int* x, int a, int b) {
	int t = x[a];
	x[a] = x[b];
	x[b] = t;
}

PObject* arrays_clone(PObject* array, int array_length) {
	PObject* new_array = malloc(array_length * sizeof(PObject));
	int i = 0;
	for (; i < array_length; i++) {
		new_array[i] = array[i];
	}
	return new_array;
}

PObject* arrays_copyOf(PObject* original, int original_length, int newLength) {
	PObject* copy = malloc(newLength * sizeof(PObject));
	int copy_length = newLength;

	int min_length;
	if (original_length < newLength)
		min_length = original_length;
	else
		min_length = newLength;

	system_arraycopy(original, original_length, 0, copy, copy_length, 0,
			min_length);

	return copy;
}

void arrays_mergeSort(PObject* src, int src_length, PObject* dest,
		int dest_length, int low, int high, int off) {
	int INSERTIONSORT_THRESHOLD = 7;

	int length = high - low;

	// Insertion sort on smallest arrays
	if (length < INSERTIONSORT_THRESHOLD) {
		arrays_insertionSort0(dest, dest_length, low, high);
	}

	// Recursively sort halves of dest into src
	int destLow = low;
	int destHigh = high;
	low += off;
	high += off;
	int mid = low + ((high - low) / 2);
	arrays_mergeSort(dest, dest_length, src, src_length, low, mid, -off);
	arrays_mergeSort(dest, dest_length, src, src_length, mid, high, -off);

	// If list is already sorted, just copy from src to dest.  This is an
	// optimization that results in faster sorts for nearly ordered lists.

	if (object_intValue(src[mid - 1]) <= object_intValue(src[mid])) {
		system_arraycopy(src, src_length, low, dest, dest_length, destLow,
				length);
		return;
	}

	// Merge sorted halves (now in src) into dest
	int i = destLow, p = low, q = mid;
	for (; i < destHigh; i++) {
		if (q >= high
				|| (p < mid
						&& (object_intValue(src[p]) <= object_intValue(src[q]))))
			dest[i] = src[p++];
		else
			dest[i] = src[q++];
	}
}

void arrays_sort(PObject* a, int a_length) {
	PObject* aux = arrays_clone(a, a_length);
	int aux_length = a_length;
	arrays_mergeSort(aux, aux_length, a, a_length, 0, a_length, 0);
}

/*
 * --------------------------------------------------------------
 * binarySearch0(int[] a, int fromIndex, int toIndex, int key)
 * --------------------------------------------------------------
 */
int arrays_binarySearch0(int* a, int a_length, int fromIndex, int toIndex,
		int key) {
	int low = fromIndex;
	int high = toIndex - 1;
	int mid;
	while (low <= high) {
		mid = low + ((high - low) / 2);
//		assert (a!=NULL);			//SAFE:0.43s
//		assert (mid>=0);			//CRASH:0.566s
//		assert (mid<a_length);  	//CRASH:0.567s
		int midVal = a[mid];
		if (midVal < key) {
			low = mid + 1;
		} else if (midVal > key) {
			high = mid - 1;
		} else {
			return mid;
		}
	}
	return -(low + 1);
}

void blast_binarySearch0(int *a, int a_length, int fromIndex, int toIndex,
		int key) {
	if (a == NULL)
		return;

	if (a_length < 0)
		return;

	if (!t_arrays_within(a_length, fromIndex, toIndex))
		return;

	if (!t_arrays_sorted(a, a_length, fromIndex, toIndex))
		return;

	int ret_val = arrays_binarySearch0(a, a_length, fromIndex, toIndex, key);

//	assert ((ret_val < 0) || (a[ret_val]==key)); 					//UNSAFE:0.788s
//	assert((ret_val >= 0) || (t_arrays_in(a, a_length, fromIndex, toIndex, key))); //UNSAFE:0.527s
}

/*
 * --------------------------------------------------------------
 * equals0(int[] a, int[] a2)
 * --------------------------------------------------------------
 */

bool ensures_equals0(int* a, int a_length, int* a2, int a2_length) {

	if (a == a2)
		return TRUE;
	if (a != NULL)
		return FALSE;
	if (a2 != NULL)
		return FALSE;

	if (a_length != a2_length)
		return FALSE;

	int i = 0;
	for (; i < a_length; i++) {
		if (a[i] != a2[i])
			return FALSE;
	}
	return TRUE;
}

bool arrays_equals0(int* a, int a_length, int* a2, int a2_length) {
	if (a == a2)
		return TRUE;

	if (a == NULL || a2 == NULL)
		return FALSE;

//	assert (a!=NULL);	//SAFE:
//	assert (a2!=NULL);	//SAFE:
	int length = a_length;
	if (a2_length != length)
		return FALSE;

	int ic = 0;
	for (; ic < length; ic++) {
//		assert (a!=NULL);			//SAFE:0.429s
//		assert (ic>=0);				//SAFE:0.216s
//		assert (ic<a_length);		//SAFE:0.294s
//		assert (a2!=NULL);			//SAFE:0.429s
//		assert (ic>=0);				//SAFE:0.203s
//		assert (ic<a2_length);		//SAFE:0.299s
		if (a[ic] != a2[ic]) {
			return FALSE;
		}
	}
	return TRUE;

}

void blast_equals0(int* a, int a_length, int* a2, int a2_length) {
	bool result = arrays_equals0(a, a_length, a2, a2_length);

//	assert(result == ensures_equals0(a, a_length, a2, a2_length); //UNSAFE:0.31s
}

/*
 * --------------------------------------------------------------
 * fill0(int[] a, int val)
 * --------------------------------------------------------------
 */

void arrays_fill0(int* a, int a_length, int val) {
	int ic = 0;
	//assert (a!=NULL);	//LOOP1	//SAFE:0.345s
	for (; ic < a_length; ic++) {
//		assert(a != NULL);	//SAFE:0.287s
//		assert(ic >= 0);		//SAFE:0.354s
//		assert(ic < a_length);	//SAFE:0.354s
		a[ic] = val;
		//assert (a!=NULL) //EXTRA:LOOP1
	}
}

void blast_fill0(int* a, int a_length, int val) {

	if (a == NULL)
		return;

	arrays_fill0(a, a_length, val);
//	assert(t_arrays_eq_value(a, 0, a_length, val) == TRUE);	//UNSAFE:0.625s
}

/*
 * --------------------------------------------------------------
 * fill1(int[] a, int fromIndex, int toIndex, int val)
 * --------------------------------------------------------------
 */

bool arrays_rangeCheck(int array_length, int fromIndex, int toIndex) {
	if (fromIndex > toIndex)
		return FALSE;
	if (fromIndex < 0)
		return FALSE;
	if (toIndex > array_length)
		return FALSE;

	return TRUE;
}
void arrays_fill1(int* a, int a_length, int fromIndex, int toIndex, int val, PErrorCode error_code) {
	//assert (a!=NULL); //SAFE:1.193s
	//assert ((0 <= fromIndex && fromIndex <= toIndex && toIndex <= a_length) || (fromIndex > toIndex) || ( fromIndex < 0 || toIndex > a_length)); //SAFE:0.37
	if (!arrays_rangeCheck(a_length, fromIndex, toIndex)) {
		(*error_code) = ERROR;
		return;
	}

	int ic = fromIndex;
	for (; ic < toIndex; ic++) {
//		assert(a != NULL);	//SAFE:1.193s
//		assert(ic >= 0); //SAFE:0.673s
//		assert(ic < a_length);	//SAFE:0.673s
		a[ic] = val;
	}

}
bool ensures_equals(int* a, int* b, int fromIndex, int toIndex) {
	int i = fromIndex;
	for (; i < toIndex; i++) {
		if (a[i] != b[i]) {
			return FALSE;
		}
	}
	return TRUE;
}

void blast_fill1(int* a, int a_length, int fromIndex, int toIndex, int val) {
	ErrorCode error_code = OK;

	if (a == NULL)
		return;

	if (!t_arrays_within(a_length, fromIndex, toIndex))
		return;

	int* old_a = (int*) malloc(a_length * sizeof(int));
	int old_a_length = a_length;

	int i = 0;
	for (; i < a_length; i++) {
		old_a[i] = a[i];
	}

	arrays_fill1(a, a_length, fromIndex, toIndex, val, &error_code);

	if (error_code==OK) {
//			assert(ensures_equals(a, old_a, 0, fromIndex));	//UNSAFE:2.98s
//			assert(t_arrays_eq_value(a, fromIndex, toIndex, val));	//UNSAFE:2.279s
//			assert(ensures_equals(a, old_a, toIndex, a_length));	//UNSAFE:1.86s
	}
}

/*
 * --------------------------------------------------------------
 * fillInteger0(Object[] a, Object val)
 * --------------------------------------------------------------
 */

void arrays_fillInteger0(PObject* array, int array_length, PObject val) {
	int ic = 0;
	//assert(array!=NULL); //LOOP1 //SAFE:0.258s
	for (; ic < array_length; ic++) {
//		assert(array != NULL);	//SAFE:0.258s
//		assert(ic >= 0); //SAFE:0.319s
//		assert(ic < array_length);	//SAFE:0.319s
		array[ic] = val;
		//assert(array!=NULL); //EXTRA:LOOP1
	}
}

void blast_fillInteger0(PObject* array, int array_length, PObject val) {
	if (array == NULL)
		return;

	arrays_fillInteger0(array, array_length, val);

//	assert(val == NULL || t_arrays_object_eq(array, 0, array_length, val));	//UNSAFE:0.751s
//	assert(val == NULL || t_arrays_object_nonnull(array, 0, array_length));	//UNSAFE:0.519s
//	assert(val != NULL || t_arrays_object_eqnull(array, 0, array_length));	//UNSAFE:0.494s
}

/*
 * --------------------------------------------------------------
 * fillInteger1(Object[] a, int fromIndex, int toIndex, Object val)
 * --------------------------------------------------------------
 */

void arrays_fillInteger1(PObject* a, int a_length, int fromIndex, int toIndex,
		PObject val, PErrorCode error_code) {
//	assert (a!=NULL); //SAFE:1.079s
//	assert ((0 <= fromIndex && fromIndex <= toIndex && toIndex <= a_length) || (fromIndex > toIndex) || ( fromIndex < 0 || toIndex > a_length)); //SAFE:0.373s
	if (!arrays_rangeCheck(a_length, fromIndex, toIndex)) {
		(*error_code) = ERROR;
		return;
	}

	int ic = fromIndex;
	for (; ic < toIndex; ic++) {
//		assert(a != NULL);	//SAFE:1.079s
//		assert(ic >= 0);//SAFE:0.665s
//		assert(ic < a_length);	//SAFE:0.665s
		a[ic] = val;
	}

}

void blast_fillInteger1(PObject* a, int a_length, int fromIndex, int toIndex,
		PObject val) {
	ErrorCode error_code = OK;

	if (a == NULL)
		return;

	if (!t_arrays_within(a_length, fromIndex, toIndex))
		return;

	PObject* old_a = (PObject*) malloc(a_length * sizeof(PObject));
	int old_a_length = a_length;

	int i = 0;
	for (; i < a_length; i++) {
		old_a[i] = a[i];
	}

	arrays_fillInteger1(a, a_length, fromIndex, toIndex, val, &error_code);

	if (error_code==OK) {
//	assert(t_arrays_object_equals_2(a, old_a, 0, fromIndex));	//UNSAFE:1.87s
//	assert(t_arrays_object_equals_2(a, old_a, toIndex, a_length));//UNSAFE:1.814s
//	assert(val == NULL || t_arrays_object_nonnull(a, fromIndex, toIndex));//UNSAFE:2.32s
//	assert(val == NULL || t_arrays_object_eq(a, fromIndex, toIndex, val));//UNSAFE:2.37s
//	assert(val != NULL || t_arrays_object_eqnull(a, fromIndex, toIndex));//UNSAFE:2.40s
	}
}

/*
 * --------------------------------------------------------------
 * hashCode0(int[] a)
 * --------------------------------------------------------------
 */
int arrays_hashCode0(int* a, int a_length) {
	if (a == NULL)
		return 0;

	int result = -1;

	int ic = 0;
//	assert(a!=NULL); //LOOP1 //SAFE:0.60s
	for (; ic < a_length; ic++) {
//		assert(a != NULL);	//SAFE:0.60s
//		assert(ic >= 0); //SAFE:0.49s
//		assert(ic < a_length);	//SAFE:0.49s
		int element = a[ic];
		result = 31 * result + element;
//		assert(a!=NULL);	//EXTRA:LOOP1
	}
	return result;
}

void blast_hashCode0(int* a, int a_length) {
	int result = arrays_hashCode0(a, a_length);
//	assert(a != NULL || result == 0);	//SAFE:0.31s
}

/*
 * --------------------------------------------------------------
 * hashCodeInteger(int[] a)
 * --------------------------------------------------------------
 */

int arrays_hashCodeInteger(PObject* array, int array_length) {
	if (array == NULL)
		return 0;

	int result = 1;
	int ic = 0;
//	assert(array!=NULL); //LOOP1 //SAFE:0.295s
	for (; ic < array_length; ic++) {
//		assert(array != NULL);	//SAFE:0.295s
//		assert(ic >= 0);  //SAFE:0.458s
//		assert( ic < array_length);//SAFE:0.458s
		PObject element = array[ic];
		if (element == NULL) {
			result = 31 * result;
		} else {
//			assert(element != NULL);	//SAFE:0.210s
			result = 31 * result + object_hashCode(element);
		}
		//	assert(array!=NULL); //LOOP1 //SAFE:0.295s
	}
	return result;
}

void blast_hashCodeInteger(PObject* array, int array_length) {
	int result = arrays_hashCodeInteger(array, array_length);
//	assert(array != NULL || result == 0);//SAFE:0.483s
}

/*
 * --------------------------------------------------------------
 * insertionSort1(int[] x, int off, int len)
 * --------------------------------------------------------------
 */
void arrays_insertionSort1(int* x, int x_length, int off, int len, PErrorCode error_code) {
	if (x == NULL) {
		(*error_code)=ERROR;
		return;
	}

//	assert(x!=NULL); //SAFE:0.237s
//	assert(x!=NULL); //SAFE:0.237s
//	assert(x!=NULL); //SAFE:0.237s
	if (off < 0 || len > x_length || len < 0 || off > x_length
			|| (off + len) > x_length) {
		(*error_code)=ERROR;
		return;
	}

	if (len == 0)
		return;

	if (len == 1)
		return;

	int high = off + len;
	int ic = off + 1;
	while (ic < high) {

//		assert(x != NULL);	//SAFE:0.237s
//		assert(ic >= 0);  //SAFE:0.676s
//		assert(ic < x_length);	//SAFE:0.676s
		int new_value = x[ic];

//		assert(x != NULL);	//SAFE:0.237s
//		assert(ic >= 0); //SAFE:0.691s
//		assert(ic < x_length);	//SAFE:0.691s
//		assert(x != NULL);	//SAFE:0.237s
//		assert((ic - 1) >= 0);  //SAFE:0.669s
//		assert((ic - 1) < x_length);	//SAFE:0.669s
		x[ic] = x[ic - 1];
		int jc = ic;
//		assert(jc <= off || x != NULL);	//LOOP1	//SAFE:0.369s
//		assert(jc <= off || ((jc - 1) >= 0);//LOOP2//SAFE:1.24s
//		assert(jc <= off || ((jc - 1) < x_length));//LOOP3//SAFE:1.24s
		while (jc > off && x[jc - 1] > new_value) {
//			assert(x != NULL);	//SAFE:0.448s
//			assert(jc >= 0);//SAFE:1.035s
//			assert(jc < x_length);//SAFE:1.035s
//			assert(x != NULL);	//SAFE:0.448s
//			assert((jc - 1) >= 0);//SAFE:0.984
//			assert((jc - 1) < x_length);//SAFE:0.984
			x[jc] = x[jc - 1];
			jc--;
//			assert(jc <= off || x != NULL);	//EXTRA:LOOP1
//			assert(jc <= off || ((jc - 1) >= 0); //EXTRA:LOOP2
//			assert(jc <= off || ((jc - 1) < x_length)); //EXTRA:LOOP3
		}

//		assert(x != NULL);	//SAFE:0.6s
//		assert(jc >= 0);	//SAFE:2.299s
//		assert(jc < x_length);	//SAFE:2.299s
		x[jc] = new_value;
		ic++;
	}
}

void blast_insertionSort1(int* x, int x_length, int off, int len) {
	ErrorCode error_code=OK;

	if (x == NULL) {
		return;
	}

	if (len < 0) {
		return;
	}

	if (!t_arrays_within(x_length, off, off + len))
		return;

	arrays_insertionSort1(x, x_length, off, len, &error_code);

	if (error_code==OK) {
//		assert(t_arrays_sorted(x, x_length, off, off + len));	//UNSAFE:2.159s
	}
}

/*
 * --------------------------------------------------------------
 * merge0(Object[] src, Object[] dest, int low, int mid, int high, int destLow, int destHigh)
 * --------------------------------------------------------------
 */
void arrays_merge0(PObject* src, int src_length, PObject* dest, int dest_length,
		int low, int mid, int high, int destLow, int destHigh) {
	int ic = destLow;
	int p = low;
	int q = mid;

	for (; ic < destHigh; ic++) {
//		assert(src != NULL);	//SAFE:1.493s
//		assert(p >= 0);	//SAFE:3.876s
//		assert(p < src_length);	//SAFE:3.876s
//		assert(src != NULL);	//SAFE:1.493s
//		assert(q >= 0);	//SAFE:2.193s
//		assert(q < src_length);	//UNSAFE:1.283s
//		assert(src[p] != NULL);	//UNSAFE:1.329s
//		assert(src[q] != NULL);	//UNSAFE:1.385s
		if (q >= high
				|| (p < mid
						&& (object_intValue(src[p]) <= object_intValue(src[q])))) {
//			assert(dest != NULL);	//SAFE:1.522s
//			assert(ic >= 0);	//SAFE:2.247s
//			assert(ic < dest_length);	//SAFE:2.247s
//			assert(src != NULL);	//SAFE:1.493s
//			assert(p >= 0);	//SAFE:3.876s
//			assert(p < src_length);	//SAFE:3.876s
			dest[ic] = src[p];
			p++;
		} else {
//			assert(dest != NULL);	//SAFE:1.585s
//			assert(ic >= 0);	//SAFE:2.268s
//			assert(ic < dest_length);	//SAFE:2.268s
//			assert(src != NULL);	//SAFE:1.493s
//			assert(q >= 0);	//SAFE:2.193s
//			assert(q < src_length);	//SAFE:1.583s
			dest[ic] = src[q];
			q++;
		}
	}
}

void blast_merge0(PObject* src, int src_length, PObject* dest, int dest_length,
		int low, int mid, int high, int destLow, int destHigh) {
	if (src == NULL)
		return;

	if (dest == NULL)
		return;
	if (!t_arrays_object_sorted(src, low, mid))
		return;

	if (!t_arrays_object_sorted(src, mid, high))
		return;

	if (!t_arrays_within(src_length, low, high))
		return;

	if (low > mid || mid > high)
		return;

	if (!t_arrays_within(dest_length, destLow, destHigh))
		return;

	if (high - low != destHigh - destLow)
		return;

	if (!t_arrays_object_nonnull(src, low, high))
		return;

	if (!((src != dest) || (high <= destLow || destHigh <= low)))
		return;

	arrays_merge0(src, src_length, dest, dest_length, low, mid, high, destLow,
			destHigh);

//	assert(t_arrays_object_sorted(dest, destLow, destHigh));	//UNSAFE:22.443s
}

/*
 * --------------------------------------------------------------
 * quicksortPartition(int[] x, int off, int len, int v)
 * --------------------------------------------------------------
 */
PPartitionIndexes arrays_quicksortPartition(int* x, int x_length, int off, int len, int v, PErrorCode error_code) {
	int a = off;
	int b = a;
	int c = off + len - 1;
	int d = c;

	while (TRUE) {
//		assert(b > c || x != NULL);	//LOOP1	//SAFE:0.884s
//		assert(b > c || (b >= 0));	//LOOP2	//SAFE:0.979s
//		assert(b > c || (b < x_length));	//LOOP3	//SAFE:0.979s
		while (b <= c && x[b] <= v) {
//			assert(x != NULL);	//SAFE:0.983s
//			assert(b >= 0);	//SAFE:0.972s
//			assert(b < x_length);	//SAFE:0.972s
			if (x[b] == v) {
//				assert (x!=NULL);	//SAFE:1.006s
//				assert(t_arrays_in_bound(x_length, a) && t_arrays_in_bound(x_length, b));	//SAFE:32.172s
				arrays_swap(x, a, b);
				a++;
			}
			b++;
//			assert(b > c || x != NULL);	//EXTRA:LOOP1
//			assert(b > c || (b >= 0));	//EXTRA:LOOP2
//			assert(b > c || (b < x_length));	//EXTRA:LOOP3
		}

//		assert(c < b || x != NULL);	//LOOP4	//SAFE:1.053s
//		assert(c < b || (c >= 0));	//LOOP5	//SAFE:1.205s
//		assert(c < b || (c < x_length));	//LOOP6	//SAFE:1.205s
		while (c >= b && x[c] >= v) {
//			assert(x != NULL);	//SAFE:1.087s
//			assert(c >= 0);	//SAFE:1.253s
//			assert(c < x_length);	//SAFE:1.253s
			if (x[c] == v) {
//				assert (x!=NULL);	//SAFE:1.183s
//				assert(t_arrays_in_bound(x_length, c)  && t_arrays_in_bound(x_length, d) );	//SAFE:3.930s
				arrays_swap(x, c, d);
				d--;
			}
			c--;
//			assert(c < b || x != NULL);	//EXTRA:LOOP4
//			assert(c < b || (c >= 0 ));	//EXTRA:LOOP5
//			assert(c < b || (c < x_length));	//EXTRA:LOOP6
		}
		if (b > c) {
			break;
		}
//		assert (x!=NULL);	//SAFE:1.183s
//		assert(t_arrays_in_bound(x_length, b)  && t_arrays_in_bound(x_length, c) );	//SAFE:3.930s
		arrays_swap(x, b, c);
		b++;
		c--;
	}

	PPartitionIndexes partitionIndexes = malloc(sizeof(PartitionIndexes));
	if (partitionIndexes==NULL) {
		(*error_code)=ERROR;
		return NULL;
	}
//	assert (partitionIndexes!=NULL);
	partitionIndexes->a=a;
//	assert (partitionIndexes!=NULL);
	partitionIndexes->b=b;
//	assert (partitionIndexes!=NULL);
	partitionIndexes->c=c;
//	assert (partitionIndexes!=NULL);
	partitionIndexes->d=d;

	return partitionIndexes;

}

void blast_quicksortPartition(int* x, int x_length, int off, int len, int v) {
	PErrorCode error_code = OK;

	if (x == NULL)
		return;

	if (len < 0)
		return;

	if (off < 0)
		return;

	if (off + len > x_length)
		return;

	int* old_x = (int*) malloc(x_length * sizeof(int));
	int old_x_length = x_length;

	int i = 0;
	for (; i < x_length; i++) {
		old_x[i] = x[i];
	}

	PPartitionIndexes partitionIndexes = arrays_quicksortPartition(x, x_length, off, len, v, &error_code);

	if (error_code==OK) {
		int ret_a =partitionIndexes->a;
		int ret_b =partitionIndexes->b;
		int ret_c=partitionIndexes->c;
		int ret_d=partitionIndexes->d;


	//	assert(t_arrays_eq_value(x, off, ret_a, v));	//UNSAFE:0.646s
	//	assert(t_arrays_less(x, ret_a, ret_b, v));	//UNSAFE:0.702s
	//	assert(t_arrays_grt(x, ret_c + 1, ret_d + 1, v));//UNSAFE:0.652s
	//	assert(t_arrays_eq_value(x, ret_d + 1, off + len, v));//UNSAFE:0.650s
	//	assert(off <= ret_a);	//UNSAFE:0.488s
	//	assert(ret_a <= ret_b);	//UNSAFE:0.528s
	//	assert(ret_b == ret_c + 1);	//UNSAFE:0.533s
	//	assert(ret_c <= ret_d);	//UNSAFE:0.444s
	//	assert(ret_d < off + len);	//UNSAFE:0.533s
	//	assert(t_arrays_equals(x, old_x, 0, off));	//UNSAFE:1.172s
	//	assert(t_arrays_equals(x, old_x, off + len, x_length));	//UNSAFE:1.226s
	}
}
/*
 * --------------------------------------------------------------
 * vecswap(int[] x, int a, int b, int n)
 * --------------------------------------------------------------
 */
void arrays_vecswap(int* x, int x_length, int a, int b, int n) {
	int ic = 0;
	for (; ic < n; ic++) {
//		assert (x!=NULL);	//SAFE:6.32s
//		assert (t_arrays_in_bound(x_length, a+ic) && t_arrays_in_bound(x_length, b+ic));	//SAFE:22.164s
		arrays_swap(x, a + ic, b + ic);
	}
}

bool ensures_vecswap(int* x, int off_x, int* old_x, int off_old_x,
		int fromIndex, int toIndex) {

	int i = fromIndex;
	for (; i < toIndex; i++) {
		if (x[off_x + i] != old_x[off_old_x + i])
			return FALSE;
	}
	return TRUE;
}

void blast_vecswap(int* x, int x_length, int a, int b, int n) {
	if (x == NULL)
		return;

	if (n < 0)
		return;

	if (!t_arrays_within(x_length, a, a + n))
		return;

	if (!t_arrays_within(x_length, b, b + n))
		return;

	if (!(a + n <= b || b + n <= a))
		return;

	int* old_x = (int*) malloc(x_length * sizeof(int));
	int old_x_length = x_length;

	int i = 0;
	for (; i < x_length; i++) {
		old_x[i] = x[i];
	}

	arrays_vecswap(x, x_length, a, b, n);

	int min_a_b;
	if (a < b)
		min_a_b = a;
	else
		min_a_b = b;

	int max_a_b;
	if (a > b)
		max_a_b = a;
	else
		max_a_b = b;

//	assert(t_arrays_equals(x, old_x, 0, min_a_b));	//UNSAFE:3.245s
//	assert(t_arrays_equals(x, old_x, max_a_b, x_length));	//UNSAFE:2.378s
//	assert(ensures_vecswap(x, a, old_x, b, 0, n));	//UNSAFE:5.724s
//	assert(ensures_vecswap(x, b, old_x, a, 0, n));	//UNSAFE:6.032s
}
/*
 * --------------------------------------------------------------
 * insertionSort0(Oject[] dest, int low, int high)
 * --------------------------------------------------------------
 */
void arrays_insertionSort0(PObject* dest, int dest_length, int low, int high) {
//		assert(dest != NULL);	//SAFE:1.24s
	if (dest_length <= 1)
		return;

	if (low == high)
		return;

	int ic = low + 1;
	while (ic < high) {
//		assert(dest != NULL);	//SAFE:1.24s
//		assert(ic >= 0);//SAFE:2.701s
//		assert(ic < dest_length);	//SAFE:2.701s
		PObject new_value = dest[ic];

//		assert (dest!=NULL); //SAFE:1.24s
//		assert(ic >= 0);//SAFE:2.701s
//		assert(ic < dest_length);	//SAFE:2.701s
//		assert (dest!=NULL); //SAFE:1.24s
//		assert((ic - 1) >= 0);//SAFE:2.83s
//		assert((ic - 1) < dest_length);	//SAFE:2.83s
		dest[ic] = dest[ic - 1];

		int jc = ic;

//		assert(jc <= low || dest != NULL);	//LOOP1 //SAFE:1.663s
//		assert(jc <= low || ((jc - 1) >= 0);	//LOOP2 //SAFE:3.271s
//		assert(jc <= low || ((jc - 1) < dest_length));	//LOOP3 //SAFE:3.271s
//		assert(jc <= low || new_value != NULL);	//LOOP4	//UNSAFE:1.421s
//		assert(jc <= low || dest[jc-1] != NULL);	//LOOP5	//UNSAFE:1.421s

		while (jc > low
				&& object_intValue(dest[jc - 1]) > object_intValue(new_value)) {
//			assert(dest != NULL);	//SAFE:0.247s
//			assert(jc >= 0);  //SAFE:0.248s
//			assert(jc < dest_length);	//SAFE:0.248s
//			assert(dest != NULL);	//SAFE:0.247s
//			assert((jc - 1) >= 0); //SAFE:0.299s
//			assert((jc - 1) < dest_length);	//SAFE:0.299s
			dest[jc] = dest[jc - 1];
			jc--;
//			assert(jc <= low || dest != NULL);	//EXTRA:LOOP1
//			assert(jc <= low || ((jc - 1) >= 0);	//EXTRA:LOOP2
//			assert(jc <= low || ((jc - 1) < dest_length));	//EXTRA:LOOP3
//			assert(jc <= low || new_value != NULL);	//EXTRA:LOOP4
//			assert(jc <= low || dest[jc-1] != NULL);	//EXTRA:LOOP5
		}

//		assert(dest != NULL);	//SAFE:2.04s
//		assert(jc >= 0);  //SAFE:4.99s
//		assert(jc < dest_length);	//SAFE:4.99s
		dest[jc] = new_value;
		ic++;
	}
}

void blast_insertionSort0(PObject* dest, int dest_length, int low, int high) {
	if (dest == NULL)
		return;

	if (!t_arrays_within(dest_length, low, high))
		return;

	if (!t_arrays_object_nonnull(dest, low, high))
		return;

	arrays_insertionSort0(dest, dest_length, low, high);

//	assert(t_arrays_object_sorted(dest, low, high));	//UNSAFE:4s

}

